# Virtual environment on Lxplus

In the following we will discuss how to setup a python3 virtual environment (*venv*) on lxplus to use `mva-trainer`.
The setup should work ony any system with access to `cvmfs`.

First we have to setup the typical ATLAS environment:
```
setupATLAS
```
Now we setup `python3` and `ROOT` versions:
```
lsetup "python 3.9.14-x86_64-centos7"
lsetup "root 6.26.08-x86_64-centos7-gcc11-opt"
```
Next we instantiate our venv called `myenv` and *start* it:
```
python3 -m venv myenv
source myenv/bin/activate
```
The next step consists of upgrading pip and installing the required packages in the environment:
```
python3 -m pip install --no-cache-dir  --upgrade pip
python3 -m pip install --no-cache-dir --upgrade uproot==5.0.2
python3 -m pip install --no-cache-dir --upgrade pandas==1.5.2
python3 -m pip install --no-cache-dir --upgrade scikit-learn==1.2.0
python3 -m pip install --no-cache-dir --upgrade tables==3.8.0
python3 -m pip install --no-cache-dir --upgrade matplotlib==3.6.2
python3 -m pip install --no-cache-dir --upgrade pydot==1.4.2
python3 -m pip install --no-cache-dir torch==1.12.1 torchvision torchaudio -f https://data.pyg.org/whl/torch-1.12.0+cpu.html
python3 -m pip install --no-cache-dir torch-scatter torch-sparse torch-cluster torch-spline-conv torch-geometric
```

Now we check if we can access `ROOT`.
```
root --version
```
You should now see a `ROOT` command prompt showing version 6.26.08.

Now you can use this environment to run the scripts.

If you want to leave the venv do:
```
deactivate
```
The next time you are running the venv you don't need to reinstall the packages again. Instead you just have to activate it.