# Scaling of input variables

MinMax and Standard scaling are both commonly used techniques to normalize the input data before feeding it into a neural network.
Normalizing the input data is important because it helps the neural network converge faster and perform better.

## MinMax Scaling

MinMax scaling is a normalization technique that scales the input features to a fixed range of values between 0 and 1. It works by subtracting the minimum value of the feature and then dividing by the range (i.e., the difference between the maximum and minimum values). The formula for MinMax scaling is:

<img src="https://latex.codecogs.com/svg.image?X_{\text{scaled}}=\frac{X-X_{\text{min}}}{X_{\text{max}}-X_{\text{min}}}" title="MinMax scaling formula" />

where X is the input feature, X_min and X_max are the minimum and maximum values of the feature, respectively, and X_scaled is the normalized value of X.

## Standard Scaling

Standard scaling, also known as z-score normalization, is another popular normalization technique. It scales the input features so that they have a mean of 0 and a standard deviation of 1. It works by subtracting the mean of the feature and then dividing by the standard deviation. The formula for standard scaling is:

<img src="https://latex.codecogs.com/svg.image?X_{\text{scaled}}=\frac{X-\mu}{\sigma}" title="Standard scaling formula" />

where X is the input feature, mu is the mean of the feature, sigma is the standard deviation of the feature, and X_scaled is the normalized value of X.

Both MinMax scaling and standard scaling have their pros and cons, and the choice of normalization technique depends on the specific problem and the nature of the input data. In general, MinMax scaling is a good choice when the input data is known to have a fixed range, while standard scaling is a good choice when the input data is normally distributed.